#include "Error.h"
#include "Input.h"

#define MOUSE_BUFFER_SIZE 16

Input::Input(HINSTANCE inst, HWND wnd)
	: m_di(0)
	, m_mouse(0)
	, m_mouse_delta(0.f, 0.f)
	, m_lmb_pressed(false)
	, m_rmb_pressed(false)
{
	HRESULT res;

	// Create the device
	res = DirectInput8Create(inst, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&m_di, NULL);
	if (FAILED(res))
		throw Error("Error initializing DirectInput!");

	// Create the mouse device
	res = m_di->CreateDevice(GUID_SysMouse, &m_mouse, NULL);
	if (FAILED(res))
		throw Error("Error initilizing mouse input!");

	// Set the mouse data format
	res = m_mouse->SetDataFormat(&c_dfDIMouse);
	if (FAILED(res))
		throw Error("Error setting the mouse data format!");

	// Set the mouse behavior
	res = m_mouse->SetCooperativeLevel(
		wnd,
		DISCL_EXCLUSIVE | DISCL_FOREGROUND);
	if (FAILED(res))
		throw Error("Error setting the mouse cooperative level!");

	// Set the mouse buffer size
	DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = MOUSE_BUFFER_SIZE;

	res = m_mouse->SetProperty(DIPROP_BUFFERSIZE, &dipdw.diph);
	if (FAILED(res))
		throw Error("Error setting the mouse buffer size!");

	// Acquire the mouse
	Acquire();
}

Input::~Input()
{
	if (m_mouse != 0)
		m_mouse->Release();

	if (m_di != 0)
		m_di->Release();
}

void Input::Acquire()
{
	m_mouse->Acquire();
}

void Input::Poll()
{
	HRESULT res;

	m_mouse_delta = D3DXVECTOR2(0.f, 0.f);
	m_lmb_pressed = false;
	m_rmb_pressed = false;

	while (true)
	{
		// Get the device data
		DIDEVICEOBJECTDATA od;
		DWORD elements = 1;

		res = m_mouse->GetDeviceData(sizeof(DIDEVICEOBJECTDATA), &od, &elements, 0);
		if (res == DIERR_INPUTLOST)
		{
			m_mouse->Acquire();
			return;
		}
		if (FAILED(res) || elements == 0)
			return;

		// Handle the event
		switch (od.dwOfs)
		{
		case DIMOFS_X:
			m_mouse_delta.x += (float)((int)od.dwData);
			break;

		case DIMOFS_Y:
			m_mouse_delta.y += (float)((int)od.dwData);
			break;

		case DIMOFS_BUTTON0:
			if (od.dwData & 0x80)
				m_lmb_pressed = true;
			break;

		case DIMOFS_BUTTON1:
			if (od.dwData & 0x80)
				m_rmb_pressed = true;
			break;
		}
	}
}

D3DXVECTOR2 Input::Get_Mouse_Delta() const
{
	return m_mouse_delta;
}

bool Input::Left_Mouse_Pressed() const
{
	return m_lmb_pressed;
}

bool Input::Right_Mouse_Pressed() const
{
	return m_rmb_pressed;
}
