#ifndef __MATH_UTIL_H
#define __MATH_UTIL_H
#include <d3dx9.h>
#include <stdlib.h>

static inline int Abs(int v)
{
	return (v < 0) ? -v : v;
}

static inline float Abs(float v)
{
	return (v < 0.f) ? -v : v;
}

static inline float Clamp(float v, float _min, float _max)
{
	return (v < _min) ? _min : ((v > _max) ? _max : v);
}

static inline float Lerp(float from, float to, float t)
{
	return from + t * (to - from);
}

static inline float Rand_Float()
{
	return (float)((double)rand() / (double)RAND_MAX);
}

static inline float Rand_Float(float _min, float _max)
{
	return Lerp(_min, _max, Rand_Float());
}

static inline float Vec_Dist(const D3DXVECTOR3 &v1, const D3DXVECTOR3 &v2)
{
	D3DXVECTOR3 v = v2 - v1;
	return D3DXVec3Length(&v);
}

static inline D3DXVECTOR3 Vec4_To_Vec3(const D3DXVECTOR4 &v)
{
	return D3DXVECTOR3(v.x, v.y, v.z);
}

static inline D3DXVECTOR4 Vec3_To_Vec4(const D3DXVECTOR3 &v, float w)
{
	return D3DXVECTOR4(v.x, v.y, v.z, w);
}

static inline D3DXVECTOR3 Get_Mtx_Col3(const D3DXMATRIX &mtx, int col)
{
	return D3DXVECTOR3(mtx(0, col), mtx(1, col), mtx(2, col));
}

static inline D3DXVECTOR3 Get_Mtx_Row3(const D3DXMATRIX &mtx, int row)
{
	return D3DXVECTOR3(mtx(row, 0), mtx(row, 1), mtx(row, 2));
}

static inline D3DXVECTOR4 Get_Mtx_Row4(const D3DXMATRIX &mtx, int row)
{
	return D3DXVECTOR4(mtx(row, 0), mtx(row, 1), mtx(row, 2), mtx(row, 3));
}

#endif
